==> tplink-smarthome-simulator-1.9.0.tgz: package/lib/device-networking.js <==
'use strict';

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

const EventEmitter = require('events');

const dgram = require('dgram');

const net = require('net');

const debug = require('debug');

const {
  decrypt,
  decryptWithHeader
} = require('tplink-smarthome-crypto');

const UdpServer = require('./udp-server');

let log = debug('device');
let logUdp = debug('device:udp');
let logUdpErr = debug('device:udp:error');
let logTcp = debug('device:tcp');
let logTcpErr = debug('device:tcp:error');

class DeviceNetworking extends EventEmitter {
  constructor({
    device,
    model,
    port = 0,
    address = '0.0.0.0',
    responseDelay = 0
  } = {}) {
    super();
    this.device = device;
    this.model = model;
    this.port = port;
    this.address = address;
    this.responseDelay = responseDelay;
    this.serverBound = false;
    this.udpSocketBound = false;
  }

  processUdpMessage(msg, rinfo) {
    let decryptedMsg = decrypt(msg).toString('utf8');
    logUdp('[%s] UDP receiving', this.model, rinfo.port, rinfo.address);
    this.emit('data', {
      time: Date.now(),
      protocol: 'udp',
      message: decryptedMsg,
      remoteAddress: rinfo.address,
      remortPort: rinfo.port
    });
    let msgObj;

    try {
      msgObj = JSON.parse(decryptedMsg);
    } catch (err) {
      logUdpErr('JSON.parse, could not parse:', decryptedMsg);
      this.emit('error', err);
      throw err;
    }

    logUdp(msgObj);
    let {
      response,
      responseForLog
    } = this.device.processUdpMessage(msgObj);

    if (response !== undefined) {
      setTimeout(() => {
        logUdp('[%s] UDP responding, delay:%s,', this.model, this.responseDelay, rinfo.port, rinfo.address);
        logUdp(responseForLog);
        this.emit('response', {
          time: Date.now(),
          protocol: 'tcp',
          message: responseForLog,
          remoteAddress: rinfo.address,
          remortPort: rinfo.port
        });
        this.udpSocket.send(response, 0, response.length, rinfo.port, rinfo.address);
      }, this.responseDelay);
    }
  }

  processTcpMessage(msg, socket) {
    logTcp('[%s] TCP DATA', this.model, socket.remoteAddress, socket.remotePort);
    let decryptedMsg = decryptWithHeader(msg).toString('utf8');
    logTcp(decryptedMsg);
    this.emit('data', {
      time: Date.now(),
      protocol: 'tcp',
      message: decryptedMsg,
      localAddress: socket.localAddress,
      localPort: socket.localPort,
      remoteAddress: socket.remoteAddress,
      remortPort: socket.remotePort
    });
    let msgObj;

    try {
      msgObj = JSON.parse(decryptedMsg);
    } catch (err) {
      logUdpErr('JSON.parse, could not parse:', decryptedMsg);
      this.emit('error', err);
      throw err;
    }

    let {
      response,
      responseForLog
    } = this.device.processTcpMessage(msgObj);

    if (response !== undefined) {
      setTimeout(() => {
        logTcp('[%s] TCP responding, delay:%s,', this.model, this.responseDelay, socket.address());
        logTcp(responseForLog);
        this.emit('response', {
          time: Date.now(),
          protocol: 'tcp',
          message: responseForLog,
          localAddress: socket.localAddress,
          localPort: socket.localPort,
          remoteAddress: socket.remoteAddress,
          remortPort: socket.remotePort
        });
        socket.write(response);

        if (this.device.endSocketAfterResponse) {
          socket.end();
        }
      }, this.responseDelay);
    }
  }

  start() {
    var _this = this;

    return (0, _asyncToGenerator2.default)(function* () {
      return new Promise((resolve, reject) => {
        _this.server = net.createServer(socket => {
          socket.on('data', chunk => {
            _this.processTcpMessage(chunk, socket);
          });
          socket.on('error', err => {
            _this.emit('error', err);
          });
          socket.on('end', () => {
            socket.end();
          });
        });

        _this.server.on('error', err => {
          logTcpErr(err);
          reject(err);
        }); // bind to UDP then TCP and share port


        _this.udpSocket = dgram.createSocket({
          type: 'udp4',
          reuseAddr: true
        }); // Save to call removeListener on #stop()

        _this.udpCb = (msg, rinfo) => {
          _this.processUdpMessage(msg, rinfo);
        };

        UdpServer.on('message', _this.udpCb);

        _this.udpSocket.on('message', _this.udpCb);

        _this.udpSocket.on('error', exception => {
          logUdpErr(exception);

          _this.udpSocket.close();

          _this.udpSocketBound = false;
        });

        _this.udpSocket.bind(_this.port, _this.address, () => {
          _this.udpSocketBound = true;

          const udpAddress = _this.udpSocket.address();

          log('[%s] UDP bound', _this.model, udpAddress);

          _this.server.listen({
            port: udpAddress.port,
            host: _this.address
          }, () => {
            _this.serverBound = true;

            const tcpAddress = _this.server.address();

            _this.address = tcpAddress.address;
            _this.port = tcpAddress.port;
            log('[%s] TCP server bound', _this.model, tcpAddress);
            resolve(_this);
          });
        });
      });
    })();
  }

  stop() {
    var _this2 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      return new Promise((resolve, reject) => {
        if (typeof _this2.udpCb === 'function') {
          UdpServer.removeListener('message', _this2.udpCb);
        }

        if (_this2.udpSocketBound) {
          _this2.udpSocket.close();

          _this2.udpSocketBound = false;
        }

        if (_this2.serverBound) {
          _this2.server.close(() => {
            _this2.serverBound = false;
            resolve(_this2);
          });
        } else {
          resolve(_this2);
        }
      });
    })();
  }

}

module.exports = DeviceNetworking;

==> tplink-smarthome-simulator-1.10.0.tgz: package/lib/device-networking.js <==
'use strict';

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

const EventEmitter = require('events');

const dgram = require('dgram');

const net = require('net');

const debug = require('debug');

const {
  decrypt,
  decryptWithHeader
} = require('tplink-smarthome-crypto');

const UdpServer = require('./udp-server');

let log = debug('device');
let logUdp = debug('device:udp');
let logUdpErr = debug('device:udp:error');
let logTcp = debug('device:tcp');
let logTcpErr = debug('device:tcp:error');

class DeviceNetworking extends EventEmitter {
  constructor({
    device,
    model,
    port = 0,
    address = '0.0.0.0',
    responseDelay = 0
  } = {}) {
    super();
    this.device = device;
    this.model = model;
    this.port = port;
    this.address = address;
    this.responseDelay = responseDelay;
    this.serverBound = false;
    this.udpSocketBound = false;
  }

  processUdpMessage(msg, rinfo) {
    let decryptedMsg = decrypt(msg).toString('utf8');
    logUdp('[%s] UDP receiving', this.model, rinfo.port, rinfo.address);
    this.emit('data', {
      time: Date.now(),
      protocol: 'udp',
      message: decryptedMsg,
      remoteAddress: rinfo.address,
      remortPort: rinfo.port
    });
    let msgObj;

    try {
      msgObj = JSON.parse(decryptedMsg);
    } catch (err) {
      logUdpErr('JSON.parse, could not parse:', decryptedMsg);
      this.emit('error', err);
      throw err;
    }

    logUdp(msgObj);
    let {
      response,
      responseForLog
    } = this.device.processUdpMessage(msgObj);

    if (response !== undefined) {
      setTimeout(() => {
        logUdp('[%s] UDP responding, delay:%s,', this.model, this.responseDelay, rinfo.port, rinfo.address);
        logUdp(responseForLog);
        this.emit('response', {
          time: Date.now(),
          protocol: 'tcp',
          message: responseForLog,
          remoteAddress: rinfo.address,
          remortPort: rinfo.port
        });
        this.udpSocket.send(response, 0, response.length, rinfo.port, rinfo.address);
      }, this.responseDelay);
    }
  }

  processTcpMessage(msg, socket) {
    logTcp('[%s] TCP DATA', this.model, socket.remoteAddress, socket.remotePort);
    let decryptedMsg = decryptWithHeader(msg).toString('utf8');
    logTcp(decryptedMsg);
    this.emit('data', {
      time: Date.now(),
      protocol: 'tcp',
      message: decryptedMsg,
      localAddress: socket.localAddress,
      localPort: socket.localPort,
      remoteAddress: socket.remoteAddress,
      remortPort: socket.remotePort
    });
    let msgObj;

    try {
      msgObj = JSON.parse(decryptedMsg);
    } catch (err) {
      logUdpErr('JSON.parse, could not parse:', decryptedMsg);
      this.emit('error', err);
      throw err;
    }

    let {
      response,
      responseForLog
    } = this.device.processTcpMessage(msgObj);

    if (response !== undefined) {
      setTimeout(() => {
        logTcp('[%s] TCP responding, delay:%s,', this.model, this.responseDelay, socket.address());
        logTcp(responseForLog);
        this.emit('response', {
          time: Date.now(),
          protocol: 'tcp',
          message: responseForLog,
          localAddress: socket.localAddress,
          localPort: socket.localPort,
          remoteAddress: socket.remoteAddress,
          remortPort: socket.remotePort
        });
        socket.write(response);

        if (this.device.endSocketAfterResponse) {
          socket.end();
        }
      }, this.responseDelay);
    }
  }

  start() {
    var _this = this;

    return (0, _asyncToGenerator2.default)(function* () {
      return new Promise((resolve, reject) => {
        try {
          let udpAddress;
          let retryCount = 0;
          _this.server = net.createServer(socket => {
            socket.on('data', chunk => {
              _this.processTcpMessage(chunk, socket);
            });
            socket.on('error', err => {
              _this.emit('error', err);
            });
            socket.on('end', () => {
              socket.end();
            });
          });

          _this.server.on('listening', () => {
            _this.serverBound = true;

            const tcpAddress = _this.server.address();

            _this.address = tcpAddress.address;
            _this.port = tcpAddress.port;
            log('[%s] TCP server bound', _this.model, tcpAddress);
            resolve(_this);
          });

          _this.server.on('error', err => {
            logTcpErr(err);

            if (err.code === 'EADDRINUSE' && udpAddress != null && retryCount < 2) {
              retryCount += 1;
              logTcpErr('Address in use, retrying...');
              setTimeout(() => {
                _this.server.close();

                _this.server.listen({
                  port: udpAddress.port,
                  host: _this.address
                });
              }, 500);
              return;
            }

            reject(err);
          }); // bind to UDP then TCP and share port


          _this.udpSocket = dgram.createSocket({
            type: 'udp4',
            reuseAddr: true
          }); // Save to call removeListener on #stop()

          _this.udpCb = (msg, rinfo) => {
            _this.processUdpMessage(msg, rinfo);
          };

          UdpServer.on('message', _this.udpCb);

          _this.udpSocket.on('message', _this.udpCb);

          _this.udpSocket.on('error', exception => {
            logUdpErr(exception);

            _this.udpSocket.close();

            _this.udpSocketBound = false;
            reject(exception);
          });

          _this.udpSocket.bind(_this.port, _this.address, () => {
            _this.udpSocketBound = true;
            udpAddress = _this.udpSocket.address();
            log('[%s] UDP bound', _this.model, udpAddress);

            _this.server.listen({
              port: udpAddress.port,
              host: _this.address
            });
          });
        } catch (err) {
          reject(err);
        }
      });
    })();
  }

  stop() {
    var _this2 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      return new Promise((resolve, reject) => {
        if (typeof _this2.udpCb === 'function') {
          UdpServer.removeListener('message', _this2.udpCb);
        }

        if (_this2.udpSocketBound) {
          _this2.udpSocket.close();

          _this2.udpSocketBound = false;
        }

        if (_this2.serverBound) {
          _this2.server.close(() => {
            _this2.serverBound = false;
            resolve(_this2);
          });
        } else {
          resolve(_this2);
        }
      });
    })();
  }

}

module.exports = DeviceNetworking;
